// Copyright (c) 2014, Sandro Lutz. All rights reserved. Use of this source code
// is governed by a BSD-style license that can be found in the LICENSE file.

library temparus_gallery;

import 'dart:html';
import 'dart:js';
import 'dart:async';
import 'package:color/color.dart';

import 'src/caption_mode.dart';
import 'src/background_size.dart';

/**
 * An image-gallery for use on a website. This gallery supports youtube videos!
 *
 * Every div element can be used to be a TemparusGallery. You can predefine
 * items for your gallery within the div element in the following structure:
 *  <a href="link/to/your/image.jpg">
 *    <img src="link/to/your/preview/file.jpg" alt="comment" />
 *  </a>
 * 
 * These items will be loaded automatically.
 * You can adjust settings with the corresponding methods.
 **/
class TemparusGallery {
  Element _elementMain;
  Element _elementView;
  Element _buttonPrev;
  Element _buttonNext;
  Element _buttonFullscreen;
  Timer _timerMousedelay;
  bool _loop;
  bool _allowFullscreen;
  bool _fullscreen;
  String _captionMode;
  int _animationDuration; // in ms
  double _ratio;
  int _position;
  
  // style data
  Map<String, dynamic> _style;
  
  var _mouseOutStream;
  
  TemparusGallery(this._elementMain) {
    _animationDuration = 500;
    _ratio = 16.0/9;
    _position = 0;
    _loop = true;
    _fullscreen = false;
    _style = new Map<String, dynamic>();
    _style['shadowColor'] = new Color.hex('AAAAAA');
    _style['fontColor'] = new Color.hex('FFFFFF');
    _style['backgroundSize'] = BackgroundSize.CONTAIN;
    
    // hide all existing elements
    _elementMain.querySelectorAll('*').style.display = 'none';
    
    _elementMain
      ..style.position = 'relative'
      ..style.boxShadow = ('0 0 5px 0 #' + _style['shadowColor'].toHexString())
      ..style.fontSize = '1.5em'
      ..style.color = '#' + _style['fontColor'].toHexString();
    
    _elementView = new Element.div()
      ..style.position = 'relative'
      ..style.width = '100%'
      ..style.display = 'block'
      ..style.overflow = 'hidden'
      ..style.backgroundColor = ('rgba(' + _style['shadowColor'].r.toString() + ',' + _style['shadowColor'].g.toString() + ',' + _style['shadowColor'].b.toString() + ',0.5)')
      ..onMouseEnter.listen((MouseEvent event) => _showControls())
      ..onMouseMove.listen((MouseEvent event) => _showControls())
      ..onMouseLeave.listen((MouseEvent event) {
        _hideControls();
      });
  
    _elementView.children.add(new Element.ul()
      ..style.position = 'relative'
      ..style.width = '100%'
      ..style.height = '100%'
      ..style.whiteSpace = 'nowrap'
      ..style.listStyle = 'none'
      ..style.padding = '0'
      ..style.margin = '0'
      ..style.left = '0'
      ..style.transition = 'left ' + _animationDuration.toString() + 'ms ease-in-out');
         
    _buttonPrev = new Element.tag('i')
      ..classes.addAll(['fa','fa-chevron-left'])
      ..style.display = 'block'
      ..style.transition = 'color 0.2s ease-in-out, opacity 0.2s ease-in-out'
      ..style.position = 'absolute'
      ..style.cursor = 'pointer'
      ..style.zIndex = '10'
      ..style.textShadow = ('0 0 5px rgba(' + _style['shadowColor'].r.toString() + ',' + _style['shadowColor'].g.toString() + ',' + _style['shadowColor'].b.toString() + ',1)')
      ..style.left = '0.5em'
      ..style.top = '50%'
      ..style.transform = 'translateY(-50%)'
      ..style.opacity = '0'
      ..onClick.listen((MouseEvent event) => prev());
    
    _buttonNext = new Element.tag('i')
      ..classes.addAll(['fa','fa-chevron-right'])
      ..style.display = 'block'
      ..style.transition = 'color 0.2s ease-in-out, opacity 0.2s ease-in-out'
      ..style.position = 'absolute'
      ..style.cursor = 'pointer'
      ..style.zIndex = '10'
      ..style.textShadow = ('0 0 5px rgba(' + _style['shadowColor'].r.toString() + ',' + _style['shadowColor'].g.toString() + ',' + _style['shadowColor'].b.toString() + ',1)')
      ..style.right = '0.5em'
      ..style.top = '50%'
      ..style.transform = 'translateY(-50%)'
      ..style.opacity = '0'
      ..onClick.listen((MouseEvent event) => next());
    
    _buttonFullscreen = new Element.tag('i')
      ..classes.addAll(['fa','fa-expand'])
      ..style.display = 'block'
      ..style.transition = 'color 0.2s ease-in-out, opacity 0.2s ease-in-out'
      ..style.position = 'absolute'
      ..style.cursor = 'pointer'
      ..style.zIndex = '10'
      ..style.textShadow = ('0 0 5px rgba(' + _style['shadowColor'].r.toString() + ',' + _style['shadowColor'].g.toString() + ',' + _style['shadowColor'].b.toString() + ',1)')
      ..style.right = '0.5em'
      ..style.top = '0.5em'
      ..style.opacity = '0'
      ..onClick.listen((MouseEvent event) {
        if(_fullscreen) {
          document.exitFullscreen();
        } else {
          //workaround
          var element = new JsObject.fromBrowserObject(_elementView);

          if (element.hasProperty("requestFullscreen")) {
            element.callMethod("requestFullscreen");
          } else {
            List<String> vendors = ['moz', 'webkit', 'ms', 'o'];
            for (String vendor in vendors) {
              String vendorFullscreen = "${vendor}RequestFullscreen";
              if (vendor == 'moz') {
                vendorFullscreen = "${vendor}RequestFullScreen";
              }
              if (element.hasProperty(vendorFullscreen)) {
                element.callMethod(vendorFullscreen);
              }
            }
          }
        }
      });
    
    _elementView.children.addAll([_buttonPrev, _buttonNext, _buttonFullscreen]);
        
    _elementMain.children.add(_elementView);
    
    // process predefined items
    _elementMain.querySelectorAll('a').forEach((Element item) {
      _addItem(item.attributes['href'], item.querySelector('img').attributes['alt']);
      item.remove();
    });
    
    window.onResize.listen((Event event) {
      _updateRatio();
      if(document.fullscreenElement == _elementView) {
        _fullscreen = true;
        _elementView.style.backgroundColor = '#000';
        _buttonFullscreen
          ..classes.add('fa-compress')
          ..classes.remove('fa-expand');
      } else {
        _fullscreen = false;
        _elementView.style.backgroundColor = ('rgba(' + _style['shadowColor'].r.toString() + ',' + _style['shadowColor'].g.toString() + ',' + _style['shadowColor'].b.toString() + ',0.5)');
        _buttonFullscreen
          ..classes.add('fa-expand')
          ..classes.remove('fa-compress');
      }
    });
    
    captionMode = CaptionMode.AUTO;
    allowFullscreen = true;
    show(_position);
    _updateRatio();
  }
  
  void set ratio(double ratio) {
    _ratio = ratio;
  }
  get ratio => _ratio;
  
  void set allowFullscreen(bool fullscreen) {
    if(document.fullscreenEnabled)
      _allowFullscreen = fullscreen;
    else
      _allowFullscreen = false;
    
    if(_allowFullscreen) {
      _buttonFullscreen.style.display = 'block';
    } else {
      _buttonFullscreen.style.display = 'none';
    }
  }
  get allowFullscreen => _allowFullscreen;
  
  get fullscreen => _fullscreen;
  
  void set captionMode(String captionMode) {
    switch(captionMode) {
      case CaptionMode.SHOW:
        _elementView.querySelectorAll('figcaption')
          ..style.display = 'block'
          ..style.opacity = '1';
        break;
      case CaptionMode.HIDE:
        _elementView.querySelectorAll('figcaption').style.display = 'none';
        break;
      case CaptionMode.AUTO:
        _elementView.querySelectorAll('figcaption').style.display = 'block';
        break;
      default:
        return;
    }
    _captionMode = captionMode;
  }
  get captionMode => _captionMode;
  
  void next() {
    if(_position+1 >= _elementView.querySelector('ul').children.length) {
      if(_loop) {
        _position = 0;
      }
    } else {
      _position++;
    }
    show(_position);
  }
  
  void prev() {
    if(_position-1 < 0) {
      if(_loop) {
        _position = _elementView.querySelector('ul').children.length - 1;
      }
    } else {
      _position--;
    }
    show(_position);
  }
  
  void show(int position) {
    _position = (position >= 0 && position < _elementView.querySelector('ul').children.length) ? position : _position;
    _elementView.querySelector('ul').style.left = (_position*-100).toString() + '%';
  }
  
  void _showControls() {
    _elementView.querySelectorAll('i').style.opacity = '1';
    _elementView.querySelectorAll('figcaption').style.opacity = '1';
    if(_timerMousedelay != null)
      _timerMousedelay.cancel();
    _timerMousedelay = new Timer(new Duration(seconds: 3), _hideControls);
  }
  
  void _hideControls() {
    _elementView.querySelectorAll('i').style.opacity = '0';
    if(captionMode == 'auto')
      _elementView.querySelectorAll('figcaption').style.opacity = '0';
    if(_timerMousedelay != null)
          _timerMousedelay.cancel();
  }
  
  void _updateRatio() {
    _elementView.style.height = (_elementView.offsetWidth/_ratio).round().toString() + 'px';
  }
  
  void _addItem(String link, String caption) {
    // view
    Element mainElement;
    
    if(link.contains('youtube')) {
      mainElement = new Element.iframe()
        ..src = link
        ..allowFullscreen = true
        ..style.width = '100%'
        ..style.height = '100%'
        ..style.border = 'none';
    } else {
      mainElement = new Element.tag('figure')
        ..style.position = 'relative'
        ..style.width = '100%'
        ..style.height = '100%'
        ..style.margin = '0'
        ..style.padding = '0'
        ..style.overflow = 'hidden';
      
      if(caption.length > 0) {
        Element captionElement = new Element.tag('figcaption')
          ..innerHtml = caption
          ..style.display = 'block'
          ..style.position = 'absolute'
          ..style.left = '0'
          ..style.right = '0'
          ..style.bottom = '0'
          ..style.padding = '0.5em 2em'
          ..style.textAlign = 'center'
          ..style.fontSize = '1em'
          ..style.zIndex = '2'
          ..style.transition = 'opacity 0.2s ease-in-out'
          ..style.opacity = '0'
          ..style.backgroundColor = ('rgba(' + _style['shadowColor'].r.toString() + ',' + _style['shadowColor'].g.toString() + ',' + _style['shadowColor'].b.toString() + ',0.75)')
          ..style.whiteSpace = 'normal';
        mainElement.children.add(captionElement);
      }
      ImageElement imageElement = new Element.img()
        ..src = link
        ..style.position = 'absolute'
        ..style.top = '50%'
        ..style.left = '50%'
        ..style.transform = 'translateX(-50%) translateY(-50%)'
        ..style.width = 'auto'
        ..style.height = 'auto';
      
      if(_style['backgroundSize'] == BackgroundSize.COVER) {
        imageElement
          ..style.minWidth = '100%'
          ..style.minHeight = '100%';
      } else {
        imageElement
          ..style.maxWidth = '100%'
          ..style.maxHeight = '100%';
      }
      
      mainElement.children.add(imageElement);
    }
           
    _elementView.querySelector('ul').children.add(new Element.li()
      ..style.width = '100%'
      ..style.height = '100%'
      ..style.margin = '0'
      ..style.padding = '0'
      ..style.display = 'inline-block'
      ..children.add(mainElement));
  }
}